@@ -3,6 +3,6 @@ use strict;
 use warnings;
 
 package WWW::Shopify::Liquid::Filter::Escape; use base 'WWW::Shopify::Liquid::Filter';
-sub operate { my $str = $_[2]; $str =~ s/"/\\"/g; return $str; }
+sub operate { my $str = defined $_[2] ? $_[2] : ""; $str =~ s/"/\\"/g; return $str; }
 
 1;
\ No newline at end of file
