@@ -3,6 +3,6 @@ use strict;
 use warnings;
 
 package WWW::Shopify::Liquid::Filter::Remove; use base 'WWW::Shopify::Liquid::Filter';
-sub operate { my $str = $_[2]; $str =~ s/$_[3]//g; return $str; }
+sub operate { my $str = $_[2]; return '' unless defined $str; $str =~ s/$_[3]//g; return $str; }
 
 1;
\ No newline at end of file
