@@ -8,6 +8,10 @@ use base 'WWW::Shopify::Liquid::Filter';
 sub min_arguments { return 2; }
 sub max_arguments { return 2; }
 
-sub operate { my $str = $_[2]; $str =~ s/$_[3]/$_[4]/g; return $str; }
+sub operate { 
+	my $str = $_[2];
+	return undef unless defined $str;
+	$str =~ s/$_[3]/$_[4]/g;
+	return $str; }
 
 1;
\ No newline at end of file
