@@ -31,6 +31,36 @@ package WWW::Shopify::Liquid::Token::Bool;
 use base 'WWW::Shopify::Liquid::Token::Operand';
 sub process { my ($self, $hash) = @_; return $self->{core}; }
 
+package WWW::Shopify::Liquid::Token::FunctionCall;
+use base 'WWW::Shopify::Liquid::Token::Operand';
+
+use Scalar::Util qw(looks_like_number reftype blessed);
+
+sub new {
+	my $package = shift;
+	return bless {
+		line => shift,
+		method => shift,
+		self => shift,
+		arguments => [@_]
+	}, $package;
+}
+
+sub process {
+	my ($self, $hash, $action, $pipeline) = @_;
+	if ($action eq "render") {
+		die new WWW::Shopify::Liquid::Exception::Renderer::Forbidden($self->{line}) unless $pipeline->make_method_calls;
+		
+		my @arguments = map { $self->is_processed($_) ? $_ : $_->render($pipeline, $hash) } @{$self->{arguments}};
+		my $inner_self = $self->is_processed($self->{self}) ? $self->{self} : $self->{self}->render($pipeline, $hash);
+		my $method = $self->is_processed($self->{method}) ? $self->{method} : $self->{method}->render($pipeline, $hash);
+		die new WWW::Shopify::Liquid::Exception::Renderer($self->{line}, "Can't find method $method on $inner_self.") unless $inner_self && blessed($inner_self) && !ref($method);
+		return $inner_self->$method(@arguments);
+	}
+	return $self;
+	
+}
+
 package WWW::Shopify::Liquid::Token::Variable;
 use base 'WWW::Shopify::Liquid::Token::Operand';
 
@@ -53,8 +83,10 @@ sub process {
 			
 			return $self unless defined $key && $key ne '';
 			$self->{core}->[$part_idx] = $key if $self->is_processed($key) && $action eq "optimize";
-			if (defined $place) {				
-				if (reftype($place) && reftype($place) eq "HASH" && exists $place->{$key}) {
+			if (defined $place) {
+				if (blessed($place) && $place->isa('WWW::Shopify::Liquid::Resolver')) {
+					$place = $place->resolver->($place, $hash, $key);
+				} elsif (reftype($place) && reftype($place) eq "HASH" && exists $place->{$key}) {
 					$place = $place->{$key};
 				} elsif (reftype($place) && reftype($place) eq "ARRAY" && looks_like_number($key) && defined $place->[$key]) {
 					$place = $place->[$key];
@@ -68,7 +100,9 @@ sub process {
 			
 		}
 	}
-	return $unprocessed ? $self : $place;
+	return $self if $unprocessed;
+	return $place->resolver->($place, $hash) if (blessed($place) && $place->isa('WWW::Shopify::Liquid::Resolver'));
+	return $place;
 }
 sub stringify { return join(".", map { $_->stringify } @{$_[0]->{core}}); }
 
@@ -158,6 +192,12 @@ sub members { return @{$_[0]->{members}}; }
 sub new { 
 	my ($package, $line, @members) = @_;
 	
+	@members = map { $_->isa('WWW::Shopify::Liquid::Token::Variable::Named') ? (
+		WWW::Shopify::Liquid::Token::String->new($_->{line}, $_->{name}),
+		WWW::Shopify::Liquid::Token::Separator->new($_->{line}, ':'),
+		$_->{core}
+	) : $_ } @members;
+	
 	die new WWW::Shopify::Liquid::Exception::Lexer::InvalidSeparator($line) unless
 		int(grep { ($_ % 2) == 0 && $members[$_]->isa('WWW::Shopify::Liquid::Token::Separator') } (0..$#members)) == 0 &&
 		int(grep { 
@@ -247,7 +287,7 @@ sub parse_token {
 	}) if $token =~ m/^'(.*)'$/s || $token =~ m/^"(.*)"$/s;
 	return WWW::Shopify::Liquid::Token::Number->new($line, $1) if looks_like_number($token);
 	return WWW::Shopify::Liquid::Token::NULL->new() if $token eq '' || $token eq 'null';
-	return WWW::Shopify::Liquid::Token::Separator->new($line, $token) if ($token eq ":" || $token eq ",");
+	return WWW::Shopify::Liquid::Token::Separator->new($line, $token) if ($token eq ":" || $token eq "," || $token eq ".");
 	return WWW::Shopify::Liquid::Token::Array->new($line) if ($token eq "[]");
 	return WWW::Shopify::Liquid::Token::Hash->new($line) if ($token eq "{}");
 	# We're a variable. Let's see what's going on. Split along non quoted . and [ ] fields.
@@ -321,20 +361,20 @@ sub parse_expression {
 			$start_sq = $re if $rc eq "[" && $sq_level++ == 0;
 			$start_hsh = $re if $rc eq "{" && $hsh_level++ == 0;
 			# Deal with parentheses; always the highest level of operation, except when inside a square bracket.
-			if ($rc eq ")" && --$level == 0 && $sq_level == 0) {
+			if ($rc eq ")" && --$level == 0 && $sq_level == 0 && $hsh_level == 0) {
 				$start_space = $re;
 				push(@tokens, WWW::Shopify::Liquid::Token::Grouping->new($line, $self->parse_expression($line, substr($exp, $start_paren, $rs - $start_paren))));
 			}
 			--$sq_level if $rc eq "]";
 			--$hsh_level if $rc eq "}";
-			if ($level == 0 && $sq_level == 0 && $hsh_level == 0) {
+			if (($level == 0 || ($rc eq "(" && $level == 1)) && $sq_level == 0 && $hsh_level == 0) {
 				# If we're only spaces, that means we're a new a token.
-				if (defined $whitespace || $nword_op) {
+				if ($rc eq "(" || defined $whitespace || $nword_op) {
 					if (defined $start_space) {
 						my $contents = substr($exp, $start_space, $rs - $start_space);
-						push(@tokens, $self->parse_token($line, $contents)) if $contents !~ m/^\s*$/;
+						push(@tokens, $self->parse_token([$line->[0], $line->[1] + $start_space, $line->[2] + $start_space, $line->[4]], $contents)) if $contents !~ m/^\s*$/;
 					}
-					push(@tokens, $self->parse_token($line, $nword_op)) if $nword_op;
+					push(@tokens, $self->parse_token([$line->[0], $line->[1] + $start_space, $line->[2] + $start_space, $line->[4]], $nword_op)) if $nword_op;
 					$start_space = $re;
 				}
 			}
@@ -356,7 +396,7 @@ sub parse_expression {
 		($_ == 2 || !$tokens[$_-3]->isa('WWW::Shopify::Liquid::Token::Operator') || $tokens[$_-3]->{core} ne "|") &&
 		$tokens[$_-2]->isa('WWW::Shopify::Liquid::Token::Variable') && int(@{$tokens[$_-2]->{core}}) == 1 && $tokens[$_-2]->{core}->[0]->isa('WWW::Shopify::Liquid::Token::String') &&
 		$tokens[$_-1]->isa('WWW::Shopify::Liquid::Token::Separator') && $tokens[$_-1]->{core} eq ":" &&
-		($tokens[$_]->isa('WWW::Shopify::Liquid::Token::Grouping') || $tokens[$_]->isa('WWW::Shopify::Liquid::Token::Variable') || $tokens[$_]->isa('WWW::Shopify::Liquid::Token::Number') || $tokens[$_]->isa('WWW::Shopify::Liquid::Token::String'))
+		$tokens[$_]->isa('WWW::Shopify::Liquid::Token::Operand')
 	} (2..$#tokens);
 	for (@ids) { $tokens[$_] = WWW::Shopify::Liquid::Token::Variable::Named->new($line, $tokens[$_-2]->{core}->[0]->{core}, $tokens[$_]); $tokens[$_-2] = undef; $tokens[$_-1] = undef; }
 	@tokens = grep { defined $_ } @tokens;
@@ -367,144 +407,114 @@ sub parse_text {
 	my ($self, $text) = @_;
 	return () unless defined $text;
 	my @tokens = ();
-	my $start = 0;
-	my $line = 1;
-	my $column = 0;
-	my $lexing_halter = undef;
 	
-	my $start_line = 1;
-	my $start_column = 0;
+	my $line = 1;
 	my $line_position = 0;
 	
-	return (WWW::Shopify::Liquid::Token::Text->new(1, '')) if $text eq '';
+	my $lexing_halter;
+	my $open_control_tag;
+	my $open_output_tag;
+	my $open_single_quote;
+	my $open_double_quote;
+	my $strip_left;
 	
-	# No need to worry about quotations; liquid overrides everything.
-	# We specifically look for tags within tags due to lexing halters abilities to be placed pretty much anywhere.
-	# Oh god no, we have to rewrite this so nested output braces inside strings inside output braces are parsed properly.
-	#while ($text =~ m/(?:(?:{%\s*(\w+)\s*(.*?)(?:\s*%}|\s*(?={%)))|(?:{{\s*(.*?)\s*}})|(\n)|$)/sg) {
-	my $open_tag = undef;
-	my $open_output_tag = undef;
-	my $open_single_quote = undef;
-	my $open_double_quote = undef;
-	my $start_tag_strip = undef;
-	my $end_tag_strip = undef;
-	my @lines = split(/\n/, $text);
-	while ($text =~ m/(?:(?:({%\-?)|(\-?%}))|(?:(\{\{\-?)|(\-?\}\}))|(\n)|(")|(')|$)/sg) {
-		my ($start_tag, $end_tag, $start_output_tag, $end_output_tag, $newline, $double_quote, $single_quote) = ($1, $2, $3, $4, $5, $6, $7);
-		next if ($double_quote || $single_quote) && !$open_tag && !$open_output_tag;
-		my $column = $+[0] - $line_position;
-		my $position = [$start_line, $start_column, $-[0], $self->file_context];
-		my $end_position = [$start_line, $start_column, $+[0], $self->file_context];
-		
-		if ($start_output_tag || $start_tag) {
-			$start_tag_strip = index(($start_output_tag || $start_tag), "-") != -1;
-			$end_tag_strip = undef;
-		} elsif ($end_output_tag || $end_tag) {
-			$end_tag_strip = index(($end_output_tag || $end_tag), "-") != -1;
+	my $end_offset = 0;
+	my $start_position = [1, 0, 0];
+	
+	# This is a substitution instead of a simple match, because very long perl native UTF8 strings, are variable length
+	# meaning any substr, or index operations need to walk the string in order to complete. This is like an o(n^2) consequence
+	# so it's fine at smaller lengths, but at larger lengths, it's infeasible to call substr on far offsets with any significant
+	# frequency. So we eat the string as we process it. Possibly slightly slower than simple matching on byte-strings, but way
+	# faster on UTF-8, and at this point in my life, I just want consistency rather than best-possible-performance (this would because
+	# written in D -betterC if I was interested in best possible performance at this point.)
+	my $accumulator = '';
+	# Step 1. We only care about new lines (to count what line we're on), {% and {{.
+	my $count = 0;
+	use Time::HiRes qw(time);
+	my $start = time;
+	while ($text =~ s/^(.*?)(?:(\{\{\-?)|({%\-?))//s) {
+		$end_offset += $+[0];
+		# These variables are entirely boolean flags.
+		my ($start_output_tag, $start_control_tag) = ($2, $3);
+		$accumulator .= $1;
+		if ($accumulator ne '') {
+			# Count the number of lines in the accumulator, to ensure we have the approrpiate line position.
+			$line += ($accumulator =~ tr/\r?\n//);
+			$line_position = rindex($accumulator, "\n") + $start_position->[2] + 1;
 		}
+		$open_control_tag = [$line, $end_offset - length($start_control_tag) - $line_position, $end_offset - length($start_control_tag), $self->file_context] if $start_control_tag;
+		$open_output_tag =  [$line, $end_offset - length($start_output_tag) -  $line_position, $end_offset - length($start_output_tag), $self->file_context] if $start_output_tag;
+		$strip_left = index($start_control_tag || $start_output_tag, "-") != -1;
 		
-		if (!$lexing_halter) {
-			if ($double_quote || $single_quote) {
-				if (!$open_double_quote && $single_quote) {
-					$open_single_quote = $open_single_quote ? undef : $position;
-				} elsif (!$open_single_quote && $double_quote) {
-					$open_double_quote = $open_double_quote ? undef : $position;
-				}
-				next;
-			}
-			next if $open_single_quote || $open_double_quote;
+		$accumulator =~ s/^\s*// if int(@tokens) > 0 && $tokens[-1]->{strip_right};
+		$accumulator =~ s/\s*$// if $strip_left;
+		if ($accumulator ne '') {
+			# Count the number of lines in the accumulator.
+			push(@tokens, WWW::Shopify::Liquid::Token::Text->new($start_position, $accumulator));
+			$accumulator = '';
+		}
+		last if $text eq '';
+		$start_position = [$line, $end_offset - $line_position, $end_offset, $self->file_context];
+		
+		# Step 2. If we have an open tag, we parse out tokens from argument context inside that tag.
+		while ($text =~ s/(.*?)(?:(\")|(\')|(\-?%})|(\-?\}\})|(\r?\n))//s) {
+			my ($double_quote, $single_quote, $end_control_tag, $end_output_tag, $newline) = ($2, $3, $4, $5, $6);
+			$end_offset += $+[0];
 			
-			die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedTag([$line, ($-[0] - $line_position), $-[0]]) if ($end_tag && !defined $open_tag);
-			# Unbalanced outputs tags shouldn't throw a lexer exception, due to CSS rules. We just treat this as part of text.
-			# die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedOutputTag($position) if $end_output_tag && !defined $open_output_tag;	
-			next if $end_output_tag && !defined $open_output_tag;
-		}
-		if ($newline) {
-			++$line;
-			$line_position = $+[0];
-			next;
-		}
-		if ($start_tag) {
-			$open_tag = $position;
-			next;
-		}
-		if ($start_output_tag) {
-			$open_output_tag = $position;
-			next;
-		}
-		
-		my ($tag, $arguments, $output);
-		my $start_element = undef;
-		my $end_element = undef;
-		if ($end_tag && $open_tag) {
-			my $tag_line = substr($text, $open_tag->[2]+2, $-[0] - $open_tag->[2] - 2);
-			$start_element = $open_tag;
-			$end_element = $end_position;
-			$open_tag = undef;
-			die new WWW::Shopify::Liquid::Exception::Lexer::Tag($start_element, $tag_line) unless $tag_line =~ m/\s*(\w+)\s*(.*?)\s*$/s;
-			($tag, $arguments) = ($1, $2);
-		} elsif ($end_output_tag && $open_output_tag) {
-			$start_element = $open_output_tag;
-			$end_element = $end_position;						
-			$output = substr($text, $open_output_tag->[2]+2, $-[0] - $open_output_tag->[2] - 2);
-			$output =~ s/(^\s*|\s*$)//g;
-			$open_output_tag = undef;
-		} else {
-			$start_element = $position;
-			$end_element = $end_position;
-		}
-		
-		if ($tag && !$lexing_halter && exists $self->{lexing_halters}->{$tag}) {
-			$lexing_halter = $tag;
-			push(@tokens, WWW::Shopify::Liquid::Token::Text->new($position, substr($text, $start, $start_element->[2] - $start), $start_tag_strip, $end_tag_strip)) if ($start < $start_element->[2]);
-			push(@tokens, WWW::Shopify::Liquid::Token::Tag->new($position, $tag, [$self->parse_expression($position, $arguments)], $start_tag_strip, $end_tag_strip));
-			$start = $end_element->[2];
-			$start_line = $line;
-			$start_column = $start_element->[2] - $line_position;
-			$position = [$start_line, $start_column, $start_element->[2], $self->file_context];
-			$start_element = $position;
-		}
-		elsif ($tag && $lexing_halter && $tag eq "end" . $lexing_halter) {
-			$lexing_halter = undef;
-		}
-		if (!$lexing_halter) {
-			if ($start < $start_element->[2]) {
-				push(@tokens, WWW::Shopify::Liquid::Token::Text->new([$start_line, $start_column, $start, $self->file_context], substr($text, $start, $start_element->[2] - $start), $start_tag_strip, $end_tag_strip));
-				# Strip whitespace.
-				if (int(@tokens) > 1 && $tokens[-1]->isa('WWW::Shopify::Liquid::Token::Text') && ($tokens[-2]->isa('WWW::Shopify::Liquid::Token::Tag') || $tokens[-2]->isa('WWW::Shopify::Liquid::Token::Output')) && $tokens[-2]->{strip_right}) {
-					if ($tokens[-1]->isa('WWW::Shopify::Liquid::Token::Text::Whitespace')) {
-						splice(@tokens, -1, 1);
-					} else {
-						$tokens[-1]->{core} =~ s/^\s+//ms;
-					}
+			if (!$open_single_quote && !$open_double_quote && ($end_output_tag || $end_control_tag)) {
+				my $strip_right = index($end_control_tag || $end_output_tag, "-") != -1;
+				$accumulator .= $1;
+				if ($end_output_tag) {
+					die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedControlTag($open_control_tag) if $open_control_tag;
+					push(@tokens, WWW::Shopify::Liquid::Token::Output->new($open_output_tag, [$self->parse_expression($start_position, $accumulator)], $strip_left, $strip_right));
+					$open_output_tag = undef;
+				} else {
+					die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedOutputTag($open_output_tag) if $open_output_tag;
+					die new WWW::Shopify::Liquid::Exception::Lexer::Tag($start_position, $accumulator) if $accumulator !~ m/^\s*(\w+)\s*(.*?)\s*$/s;
+					push(@tokens, WWW::Shopify::Liquid::Token::Tag->new($open_control_tag, $1, [$self->parse_expression($start_position, $2)], $strip_left, $strip_right));
+					$lexing_halter = $tokens[-1] if $self->{lexing_halters}->{$1};
+					$open_control_tag = undef;
 				}
-				
-				$start_line = $line;
-				$start_column = $start_element->[2] - $line_position;
-				$position = [$start_line, $start_column, $start_element->[2], $self->file_context];
-				$start_element = $position;
-			}
-			push(@tokens, WWW::Shopify::Liquid::Token::Tag->new($start_element, $tag, [$self->parse_expression($position, $arguments)], $start_tag_strip, $end_tag_strip)) if $tag;
-			push(@tokens, WWW::Shopify::Liquid::Token::Output->new($start_element, [$self->parse_expression($position, $output)], $start_tag_strip, $end_tag_strip)) if $output;
-			$start = $end_element->[2];
-			$start_line = $line;
-			$start_column = $column;
-		}
-		# Strip whitespace.
-		if ($tag && $start_tag_strip && int(@tokens) > 1 && $tokens[-2]->isa('WWW::Shopify::Liquid::Token::Text')) {
-			if ($tokens[-2]->isa('WWW::Shopify::Liquid::Token::Text::Whitespace')) {
-				splice(@tokens, -2, 1);
+				$accumulator = '';
+				$start_position = [$line, $end_offset - $line_position, $end_offset, $self->file_context];
+				last;
 			} else {
-				$tokens[-2]->{core} =~ s/\s+$//ms;
+				if ($newline) {					
+					++$line;
+					$line_position = $end_offset;
+				}
+				$open_single_quote = !$open_single_quote if $single_quote && !$open_double_quote;
+				$open_double_quote = !$open_double_quote if $double_quote && !$open_single_quote;
+				$accumulator .= $&;
+			}
+		}
+		# For lexing halters, consume everything up to the next end tag explicitly.
+		if ($lexing_halter) {
+			my $core = $lexing_halter->{tag};
+			while ($text =~ s/(.*?)(?:({%\-?\s*end$core\s*\-?%})|(\r?\n))//s) {
+				if ($3) {
+					++$line;
+					$line_position = $end_offset;
+					$accumulator .= $1 . $3;
+				} else {
+					$accumulator .= $1;
+					push(@tokens, WWW::Shopify::Liquid::Token::Text->new($start_position, $accumulator)) if $accumulator ne '';
+					push(@tokens, WWW::Shopify::Liquid::Token::Tag->new($start_position, "end$core", []));
+					$accumulator = '';
+					$lexing_halter = undef;
+					last;
+				}
 			}
 		}
-		
 	}
 	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedSingleQuote($open_single_quote) if $open_single_quote;
 	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedDoubleQuote($open_double_quote) if $open_double_quote;
-	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedTag($open_tag) if $open_tag;
-	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedLexingHalt() if $lexing_halter;
+	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedLexingHalt($lexing_halter) if $lexing_halter;
+	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedControlTag($open_control_tag) if $open_control_tag;
 	die new WWW::Shopify::Liquid::Exception::Lexer::UnbalancedOutputTag($open_output_tag) if $open_output_tag;
+	$accumulator .= $text;
+	$accumulator =~ s/^\s*// if int(@tokens) > 0 && $tokens[-1]->{strip_right};
+	push(@tokens, WWW::Shopify::Liquid::Token::Text->new($start_position, $accumulator)) if $accumulator ne '';
 	return @tokens;
 }
 
@@ -515,17 +525,20 @@ sub unparse_token {
 	return "'" . do { $token =~ s/'/\\'/g; $token } . "'" if !blessed($token) && !looks_like_number($token);
 	return $token unless blessed($token);
 	return "null" if $token->isa('WWW::Shopify::Liquid::Token::NULL');
+	return '{}' if ($token->isa('WWW::Shopify::Liquid::Token::Hash'));
+	return '[]' if ($token->isa('WWW::Shopify::Liquid::Token::Array'));
 	sub translate_variable {
 		my ($self, $token) = @_;
 		my $variable = join(".", map { 
 			!$self->is_processed($_) ? (
-				$_->isa('WWW::Shopify::Liquid::Token::Variable') ? ('[' . translate_variable($self, $_) . ']') : $_->{core}
+				$_->isa('WWW::Shopify::Liquid::Token::Variable') ? ('[' . translate_variable($self, $_) . ']') : 
+					($_->isa('WWW::Shopify::Liquid::Token::Variable::Processing') ? $_->{core}->name : $_->{core})
 			) : $_
 		} @{$token->{core}});
 		$variable =~ s/\.\[/\[/g;
 		return $variable;
 	}
-	
+	return $token->{name} . ":" . $self->unparse_token($token->{core}) if ($token->isa('WWW::Shopify::Liquid::Token::Variable::Named'));
 	return translate_variable($self, $token) if $token->isa('WWW::Shopify::Liquid::Token::Variable');
 	return "(" . join("", map { $self->unparse_token($_); } @{$token->{members}}) . ")" if $token->isa('WWW::Shopify::Liquid::Token::Grouping');
 	return "'" . $token->{core} . "'" if $token->isa('WWW::Shopify::Liquid::Token::String');
