@@ -6,6 +6,6 @@ package WWW::Shopify::Liquid::Operator::Less;
 use base 'WWW::Shopify::Liquid::Operator';
 sub symbol { return '<'; }
 sub priority { return 5; }
-sub operate { return ($_[2] cmp $_[3]) == -1; }
+sub operate { return ($_[0]->ensure_numerical($_[3]) <=> $_[0]->ensure_numerical($_[4])) == -1; }
 
 1;
\ No newline at end of file
