@@ -5,7 +5,7 @@ use warnings;
 package WWW::Shopify::Liquid::Operator::Multiply;
 use base 'WWW::Shopify::Liquid::Operator';
 sub symbol { return '*'; }
-sub priority { return 4; }
-sub operate { return $_[2] * $_[3]; }
+sub priority { return 10; }
+sub operate { return $_[0]->ensure_numerical($_[3]) * $_[0]->ensure_numerical($_[4]); }
 
 1;
\ No newline at end of file
