@@ -46,26 +46,26 @@ sub interpret_inner_tokens {
 }
 
 sub render {
-	my ($self, $hash) = @_;
+	my ($self, $renderer, $hash) = @_;
 	my $arguments = $self->{arguments}->[0];
-	$arguments = $arguments->render($hash) if !$self->is_processed($arguments);
+	$arguments = $arguments->render($renderer, $hash) if !$self->is_processed($arguments);
 	my $path = $self->{paths}->{$arguments} ? $self->{paths}->{$arguments} : $self->{else};
-	$path = $path->render($hash) if $path && !$self->is_processed($path);
+	$path = $path->render($renderer, $hash) if $path && !$self->is_processed($path);
 	return defined $path ? $path : '';
 }
 
 sub optimize {
-	my ($self, $hash) = @_;
-	$self->{arguments}->[0] = $self->{arguments}->[0]->optimize($hash) if !$self->is_processed($self->{arguments}->[0]);
+	my ($self, $optimizer, $hash) = @_;
+	$self->{arguments}->[0] = $self->{arguments}->[0]->optimize($optimizer, $hash) if !$self->is_processed($self->{arguments}->[0]);
 	my $key = $self->{arguments}->[0];
 	if ($self->is_processed($key)) {
 		my $path = exists $self->{paths}->{$key} ? $self->{paths}->{$key} : $self->{else};
-		return $self->is_processed($path) ? $path : $path->optimize($hash);
+		return $self->is_processed($path) ? $path : $path->optimize($optimizer, $hash);
 	}
 	else {
-		$self->{else} = $self->{else}->optimize($hash) if !$self->is_processed($self->{else});
+		$self->{else} = $self->{else}->optimize($optimizer, $hash) if !$self->is_processed($self->{else});
 		for (grep { $self->is_processed($self->{paths}->{$_}) } keys(%{$self->{paths}})) {
-			$self->{paths}->{$_} = $self->{paths}->{$_}->optimize($hash);
+			$self->{paths}->{$_} = $self->{paths}->{$_}->optimize($optimizer, $hash);
 		}
 	}
 	return $self;
